local myname, ns = ...

--[[
Notes:

Disturbed Earth
37273620, quest 82026, waxy lump
41466045, catching wax fired into the air
40776026, spawned enemy (229809)
47616347, spawned enemy (216537)
47156330, spawned waxy lump
46676510, spawned waxy lump
51006859, ambush
50776694, squashable grubs
23695880, spawned enemy (216288)
60133139, spawned enemy (216537)
53836700
66503847
64443404
66252501
55555175

Worldsoul memories (vignette 6358)
60686749
]]

local SHADOWPHASE = ns.Class{
    __classname="ShadowPhase",
    __parent=ns.conditions.Condition,
    Label = function(self)
        local shadowed = "{spell:131233:Shadowed}"
        if self:Matched() then
            return shadowed
        else
            -- "%s in %s"
            return WARDROBE_TOOLTIP_ENCOUNTER_SOURCE:format(shadowed, self:Duration(self:NextSpawn()))
        end
    end,
    Matched = function(self)
        -- if it's more than 2.5 hours away, we must be during the current event
        return self:NextSpawn() > (3600 * 2.5)
    end,
    NextSpawn = function(self)
        -- Shadow event is one hour after the daily reset, then repeating
        -- every three hours; each time it lasts for 30 minutes.
        return (GetQuestResetTime() + 3600) % 10800
    end,
    Duration = function(self, seconds)
        if seconds > 3600 then
            return COOLDOWN_DURATION_HOURS:format(floor(seconds / 3600)) .. " " .. COOLDOWN_DURATION_MIN:format(floor((seconds % 3600) / 60))
        end
        return COOLDOWN_DURATION_MIN:format(floor(seconds / 60))
    end
}()

ns.RegisterPoints(ns.HALLOWFALL, {
    [11091678] = ns.Class{
        label="{spell:452526:Beledar's Influence}",
        texture_light = ns.atlas_texture("Mobile-Jewelcrafting", {r=1, g=1, b=0.5}),
        texture_dark = ns.atlas_texture("Mobile-Jewelcrafting", {r=0.75, g=0, b=1}),
        scale=5,
        __get={
            note=function(self) return SHADOWPHASE:Label() .. "\nBeledar switches from light to dark for 30 minutes every 3 hours." end,
            texture=function(self)
                return SHADOWPHASE:Matched() and self.texture_dark or self.texture_light
            end,
        },
        group="beledar",
    }(),
})

-- Treasures

ns.RegisterPoints(ns.HALLOWFALL, {
    [41775829] = { -- Caesper
        criteria=69692,
        quest=83263,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150),
            225639, -- Recipe: Exquisitely Eviscerated Muscle
            225592, -- Exquisitely Eviscerated Muscle
        },
        active=ns.conditions.Item(225238), -- Meaty Haunch
        related={
            [69254397]={label="{npc:217645:Torran Dellain}", note="Buy {item:225238:Meaty Haunch}", inbag=225238, minimap=true,},
        },
        note="Bring {item:225238:Meaty Haunch} from {npc:217645:Torran Dellain}, give to {npc:225948:Caesper}, follow to the treasure",
        level=73,
        vignette=6367, -- 6368 after you feed, 6366 for Disturbed Lynx Treasure
    },
    [55135193] = { -- Smuggler's Treasure
        criteria=69693,
        quest=83273,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150),
            226021, -- Jar of Pickles
        },
        note="Get the key from the {npc:226025:Dead Arathi} below",
        nearby={55425164},
        level=73,
        vignette=6370,
    },
    [59525966] = { -- Dark Ritual
        criteria=69694,
        quest=83284,
        loot={225693}, -- Shadowed Essence
        note="In cave; use the book, defeat the summoned monsters",
        level=73,
        vignette=6372,
    },
    [40015112] = { -- Arathi Loremaster
        criteria=69695,
        quest=83298,
        loot={{225659, toy=true}},
        note="Answer riddles from {npc:221630:Ryfus Sacredpyr}; you need to find the books for {achievement:40622:Biblo Archivist} for the correct answers to appear",
        level=73, -- not to talk to him, but to get any of the books for answers...
        vignette=6373,
    },
    [55796954] = { -- Jewel of the Cliffs
        criteria=69697,
        quest=81971,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150),
            224580, -- Massive Sapphire Chunk
        },
        note="High up on the rocks",
        level=75,
        vignette=6174,
    },
    [30223877] = { -- Priory Satchel
        criteria=69698,
        quest=81972,
        loot={
            224578, -- Arathor Courier's Satchel
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150),
        },
        level=75,
        note="Hanging from the cathedral",
    },
    [50061382] = { -- Lost Necklace
        criteria=69699,
        quest=81978,
        loot={224575}, -- Lightbearer's Pendant
        level=75,
        vignette=6177, -- Lost Memento
    },
    [58392716] = { -- Illuminated Footlocker
        criteria=69701,
        quest=81468,
        loot={{224552, toy=true}},
        note="In cave. Catch falling glimmers from {npc:220703:Starblessed Glimmerfly} until you get {spell:442529:Glimmering Illumination}",
        level=73,
        vignette=6098,
    },
    [76775383] = { -- Spore-covered Coffer
        criteria=69702,
        quest=79275,
        loot={}, -- alchemy mats
        note="In cave",
        level=73,
        vignette=5989,
    },
    [45904510] = { -- Sky-Captains' Sunken Cache
         -- I believe the four captains are random, but the quest ids I got from the four I talked to were 82012, 82026, 82024, and 82025. ~Marthammor
        criteria=69700,
        quest=82005,
        loot={
            {224554, toy=true}, -- Silver Linin' Scepter
        },
        active={ns.conditions.QuestComplete(82012), ns.conditions.QuestComplete(82026), ns.conditions.QuestComplete(82024), ns.conditions.QuestComplete(82025)},
        note="Talk to four skyship captains flying around the zone to make this appear",
        routes={
            {45904510, 62754689, highlightOnly=true},
            {45904510, 49535554, highlightOnly=true},
            {45904510, 54821161, highlightOnly=true},
            {45904510, 27633461, highlightOnly=true},
        },
        atlas="VignetteLootElite", scale=1.2,
        vignette=6181, -- Sunken Cache
    },
}, {
    achievement=40848,
})

ns.RegisterPoints(ns.HALLOWFALL, {
    [62551633] = {
        label="Crabber Supplies",
        quest=84342,
        loot={
            ns.rewards.Currency(ns.CURRENCY_RESONANCE, 5),
            206350, -- Radiant Remnant
            212242, -- Cavedweller's Delight
            225814, -- Feather-on-a-Stick
        },
    },
})

ns.RegisterPoints(ns.HALLOWFALL, {
    [62754689] = {
        label="{npc:222333:Sky-Captain Dornald}, The Mighty Lynx",
        quest=82012,
        route={
            58174030, 55013755, 53543627, 53343609, 52253524, 52033507, 48913292, 48503264, 48273252, 48043235,
            47813233, 47573235, 47313239, 47083247, 46873259, 46663276, 46433299, 46253319, 46053337, 45393392,
            45193414, 45023438, 44863464, 44563522, 44293587, 44103648, 43584116, 43594153, 43664222, 43724256,
            46154846, 46324869, 46894935, 47524996, 48185051, 49525136, 49995156, 50225164, 50465171, 51415189,
            52145196, 54155173, 55395149, 56085131, 56745109, 59335028, 59555018, 61004968, 61234958, 61464941,
            61684921, 61884898, 62064873, 62224848, 62364821, 62484791, 62594759, 62694725, 62754689, 62784650,
            62774615, 62724584, 62634552, 62524519, 62394488, 62074425, 61744370, 61564345, 61164299, 59894181,
            59674159, 59274123,
            loop=true,
            r=1, g=0.6, b=0.6,
        },
    },
    [49535554] = {
        label="{npc:222311:Sky-Captain Clairmonte}, The Embers",
        quest=82024,
        route={
            40034513, 35694513, 34024547, 33354577, 32914604, 32454639, 32064674, 30944759, 30554798, 30394824,
            30254852, 30134883, 29944950, 29874986, 29755094, 29745129, 29805244, 30135458, 30235496, 30365530,
            30515558, 30675582, 31025625, 31195653, 31375675, 31565693, 31975725, 32185738, 32415750, 34565838,
            34785844, 35275853, 37185924, 37875947, 39476029, 42336237, 42566253, 43036280, 43956320, 44406341,
            44656348, 44886350, 45396346, 45886337, 46136330, 46376321, 46596307, 46806290, 47586208, 47946157,
            48226097, 49155797, 49275725, 49535554, 49705372, 49765297, 49745262, 49635196, 49425130, 49285097,
            48684983, 46614717, 45784650, 44184565, 43934557, 43694551, 43454547, 42094520,
            loop=true,
            r=0.6, g=1, b=0.6,
        },
    },
    [54821161] = {
        label="{npc:222337:Sky-Captain Onaro}, The Resolute Flame",
        quest=82026,
        route={
            52392748, 55451614, 55501578, 55521541, 55511504, 55481467, 55431432, 54821161, 54581100, 54301042,
            53780961, 53260901, 52870863, 52640844, 52380826, 50330731, 49320705, 47610718, 46640748, 46420758,
            46200770, 45990784, 45790800, 45590820, 45230868, 45060896, 43801212, 43361643, 43361747, 43772211,
            44142385, 46012782, 46692883, 47032919, 47783007, 47973026, 48193044, 48403058, 48633070, 49133090,
            49833111, 50093114, 50343111, 50583102, 50803088, 51353033, 51503014, 51622987,
            loop=true,
            r=0.6, g=0.6, b=1,
        },
    },
    [27633461] = {
        label="{npc:222323:Sky-Captain Aerthin}, The Righteous Hammer",
        quest=82025,
        route={
            35122517, 37532340, 38032309, 38232293, 38592260, 38802244, 39042234, 39262227, 39752216, 40762205,
            40992205, 41222207, 41462214, 41682225, 41882242, 42072262, 42252287, 44342661, 44672758, 45072932,
            45313072, 45363110, 45413211, 45373392, 45353430, 45223538, 45153573, 45103646, 44883747, 44793784,
            44693816, 44563845, 44403871, 44223896, 43024079, 42844105, 41444245, 41234259, 40534295, 39834322,
            38184362, 37704364, 34284336, 33814327, 33594321, 32444276, 30744216, 30314186, 29874149, 28704021,
            28523996, 28043912, 27793851, 27703819, 27563751, 27503683, 27493611, 27533536, 27633461, 27783395,
            28113304, 28553216, 29583053, 32802690,
            loop=true,
            r=0.6, g=1, b=0.1,
        },
    },
}, {
    achievement=40848,
    criteria=69700,
    atlas="Vehicle-Air-Occupied", scale=1.5,
    note="Find the ship on the route and talk to the captain",
})

-- Illusive Kobyss Lure (Treasures)
ns.RegisterPoints(ns.HALLOWFALL, {
    [55362720] = {label="{npc:215653:Kobyss Shadeshaper}: {item:225554:Sunless Lure}",},
    [47611854] = {label="{npc:213622:Murkfin Depthstalker}: {item:225558:Murkfin Lure}",},
    [50655037] = {label="{npc:215243:Hungering Shimmerfin}: {item:225559:Hungering Shimmerfin}",},
    [34965465] = {label="{npc:213406:Ragefin Necromancer}: {item:225560:Ragefin Necrostaff}",},
}, {
    achievement=40848, -- Treasures
    criteria=69696,
    quest=83299,
    atlas="Vehicle-TempleofKotmogu-CyanBall", scale=1.2,
    loot={{225641, toy=true}}, -- Illusive Kobyss Lure
    note="Gather and combine {item:225554:Sunless Lure}, {item:225558:Murkfin Lure}, {item:225559:Hungering Shimmerfin}, {item:225560:Ragefin Necrostaff}",
})

-- Biblo Archivist
ns.RegisterPoints(ns.HALLOWFALL, {
    [48153959] = {criteria=68954, loot={225212}, note="Needed for {achievement:40848.69695:Arathi Loremaster}"}, -- The Big Book of Arathi Idioms
    [43904997] = {criteria=68955, loot={225217}}, -- 500 Dishes Using Cave Fish and Mushrooms
    [69344394] = {criteria=68957, loot={225207}, note="Needed for {achievement:40848.69695:Arathi Loremaster}"}, -- Care and Feeding of the Imperial Lynx
    [68684159] = {criteria=68958, loot={225206}}, -- Light's Gambit Playbook
    [57815182] = {criteria=68960, loot={225208}, note="By the shore"}, -- From the Depths They Come
    [48756472] = {criteria=68961, loot={225216}, note="Needed for {achievement:40848.69695:Arathi Loremaster}", vignette=6374}, -- Palawltar's Codex of Dimensional Structure
    [64182812] = {criteria=68963, loot={225204}, note="Needed for {achievement:40848.69695:Arathi Loremaster}"}, -- Shadow Curfew Guidelines
    [59802203] = {criteria=68965, loot={225205}}, -- Shadow Curfew Journal
    [70225684] = {criteria=68967, loot={225215}, note="Needed for {achievement:40848.69695:Arathi Loremaster}"}, -- The Song of Renilash
    [56586518] = {criteria=68968, loot={225203}, note="In the ship. Needed for {achievement:40848.69695:Arathi Loremaster}"}, -- Beledar- The Emperor's Vision
    -- [] = {criteria=69729, loot={228457}}, -- Lightspark Grade Book
}, {
    achievement=40622,
    texture=ns.atlas_texture("profession", {r=0, g=1, b=1}),
    minimap=true,
    level=73,
})

-- The Missing Lynx
ns.RegisterPoints(ns.HALLOWFALL, {
    [60426022] = {criteria=68975, npc=220720}, -- Magpie
    [42695387] = {criteria={68998, 68999},}, -- Evan + Emery
    [42255378] = {criteria={69000, 69010},}, -- Jinx + Gobbo
    [69274372] = {criteria={69001, 69002},}, -- Moog, Iggy
    [63302940] = {criteria=7, note="Light the lesser keyflame"}, -- Nightclaw
    [63262811] = {criteria=8, note="Light the blooming keyflame"}, -- Shadowpouncer
    [63792932] = {criteria=9, note="Light the blooming keyflame"}, -- Purrlock
    [61193054] = {criteria=10,}, -- Miral Murder-Mittens
    [64441857] = {criteria={11, 12},}, -- Fuzzy, Furball
    [61922081] = {criteria=13,}, -- Dander
}, {
    achievement=40625,
    atlas="WildBattlePet", color={r=0.75, g=1, b=0},
    minimap=true,
})

-- Rares

ns.RegisterPoints(ns.HALLOWFALL, {
    [23005922] = { -- Lytfang the Lost
        criteria=69710,
        quest=81756, -- 84063
        npc=221534,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84063}),
            221207, -- Den Mother's Chestpiece
            221246, -- Fierce Beast Staff
            221251, -- Bestial Underground Cleaver
            221265, -- Charm of the Underground Beast
        },
        vignette=6145,
    },
    [63452859] = { -- Moth'ethk
        criteria=69719,
        quest=82557, -- 84051
        npc=206203,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84051}),
            221240, -- Nerubian Stagshell Gouger
            221252, -- Nerubian Slayer's Claymore
            221263, -- Nerubian Venom-Tipped Dart
            223924, -- Chitin-Inscribed Vest
        },
        vignette=5958,
        note="Objective of {questname:76588}",
    },
    [44011639] = { -- The Perchfather
        criteria=69711,
        quest=81791, -- 84064
        npc=221648,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84064}),
            221229, -- Perchfather's Cuffs
            221246, -- Fierce Beast Staff
            221247, -- Cavernous Critter Shooter
            221251, -- Bestial Underground Cleaver
            221265, -- Charm of the Underground Beast
        },
        vignette=6151,
    },
    [56466897] = { -- The Taskmaker
        criteria=69708,
        quest=80009, -- 84061
        npc=218444,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84061}),
            221215, -- Taskmaster's Mining Cap
            221240, -- Nerubian Stagshell Gouger
            221252, -- Nerubian Slayer's Claymore
            221263, -- Nerubian Venom-Tipped Dart
        },
        vignette=6033,
    },
    [31205464] = { -- Grimslice
        criteria=69706,
        quest=81761,
        npc=221551,
        loot={
            223397, -- Abyssal Hunter's Girdle
            223398, -- Abyssal Hunter's Sash
            223399, -- Abyssal Hunter's Chain
            223400, -- Abyssal Hunter's Cinch
        },
        route={
            31205464, 33235598, 32725814, 34135728, 34525751, 35085894, 35655746, 36495657, 36945464,
            36555280, 35625156, 35055029, 34555186, 34135204, 32725119, 33235334,
            r=0, g=1, b=1,
            loop=true,
        },
        vignette=6146,
        note="Patrols clockwise",
    },
    [43622993] = { -- Strength of Beledar
        criteria=69713,
        quest=81849, -- 84066
        npc=221690,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84066}),
            221216, -- Bruin Strength Legplates
            221246, -- Fierce Beast Staff
            221247, -- Cavernous Critter Shooter
            221251, -- Bestial Underground Cleaver
            221265, -- Charm of the Underground Beast
            221508, -- Pelt of Beledar's Strength
        },
        vignette=6153,
    },
    [57046436] = { -- Ixlorb the Spinner
        criteria=69704,
        quest=80006,
        npc=218426,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150),
            223374, -- Nerubian Weaver's Gown
            223379, -- Nerubian Weaver's Chestplate
            223380, -- Nerubian Weaver's Chainmail
            223381, -- Nerubian Weaver's Vest
            223100, -- Pattern: Vambraces of Deepening Darkness
        },
        vignette=6032, -- Ixlorb the Weaver
    },
    [62401320] = { -- Murkspike
        criteria=69728,
        quest=82565,
        npc=220771,
        loot={
            221233, -- Deephunter's Bloody Hook
            221234, -- Tidal Pendant
            221248, -- Deep Terror Carver
            221255, -- Sharpened Scalepiercer
            223934, -- Makrura's Foreboding Legplates
        },
        vignette=6123,
        note="Objective of {questname:76588}",
    },
    [64663172] = { -- Deathpetal
        criteria=69721,
        quest=82559, -- 84053
        npc=206184,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84053}),
            221250, -- Creeping Lasher Machete
            221253, -- Cultivator's Plant Puncher
            221264, -- Fungarian Mystic's Cluster
            223005, -- String of Fungal Fruits
            223927, -- Vinewrapped Leather Tunic
        },
        vignette=6078,
        note="Objective of {questname:76588}",
    },
    [72136436] = { -- Deepfiend Azellix
        criteria=69703,
        quest=80011,
        npc=218458,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150),
            223393, -- Deepfiend Spaulders
            223394, -- Deepfiend Pauldrons
            223395, -- Deepfiend Shoulderpads
            223396, -- Deepfiend Shoulder Shells
        },
        vignette=6035,
    },
    [64051911] = { -- Duskshadow
        criteria=69724,
        quest=82562, -- 84056
        npc=221179,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84056}),
            223918, -- Specter Stalker's Shotgun
            223919, -- Abducted Lawman's Gavel
            223936, -- Shadow Bog Trousers
        },
        vignette=6122,
        note="Objective of {questname:76588}",
    },
    [36807210] = { -- Funglour
        criteria=69707,
        quest=81881,
        npc=221767,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150),
            223377, -- Ancient Fungarian's Fingerwrap
        },
        vignette=6157,
    },
    [35953546] = { -- Sir Alastair Purefire
        criteria=69714,
        quest=81853, -- 84067
        npc=221708,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84067}),
            221241, -- Priestly Agent's Knife
            221245, -- Righteous Path Treads
        },
        vignette=6154,
    },
    [43410990] = { -- Horror of the Shallows
        criteria=69712,
        quest=81836, -- 84065
        npc=221668,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84065}),
            221211, -- Grasp of the Shallows
            221233, -- Deephunter's Bloody Hook
            221234, -- Tidal Pendant
            221248, -- Deep Terror Carver
            221255, -- Sharpened Scalepiercer
        },
        vignette=6152,
        note="Very long patrol",
        route={
            43410990, 43870879, 44520774, 45250767, 45970726, 45540662, 44870677, 44270749, 43710858, 41631452,
            41391580, 41051714, 40501821, 39731909, 36652173, 33992545, 33422650, 32912763, 31783130, 30933154,
            29993162, 29123191, 28213204, 27343238, 26553287, 26513416, 26813550, 27983757, 28633853, 29403934,
            30173998, 30764092, 30984221, 30594339, 29814381, 27194486, 26364534, 25664611, 24954700, 23314830,
            23274858, 22464885, 20774968, 19904976, 19565105, 20285138, 20865040, 21614971, 22474926,
            r=0,g=0,b=1,
        },
    },
    [73405259] = { -- Sloshmuck
        criteria=69709,
        quest=79271, -- 84062
        npc=215805,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84062}),
            221223, -- Bog Beast Mantle
            221250, -- Creeping Lasher Machete
            221253, -- Cultivator's Plant Puncher
            221264, -- Fungarian Mystic's Cluster
            223005, -- String of Fungal Fruits
        },
        vignette=5988,
    },
    [52132682] = { -- Murkshade
        criteria=69705,
        quest=80010,
        npc=218452,
        loot={
            223382, -- Murkshade Grips
            223383, -- Murkshade Handguards
            223384, -- Murkshade Gloves
            223385, -- Murkshade Gauntlets
        },
        vignette=6034,
        note="Underwater",
    },
    [67562316] = { -- Croakit
        criteria=69722,
        quest=82560, -- 84054
        npc=214757,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84054}),
            221246, -- Fierce Beast Staff
            221247, -- Cavernous Critter Shooter
            221251, -- Bestial Underground Cleaver
            221265, -- Charm of the Underground Beast
            223938, -- Marsh Hopper's Spaulders
        },
        vignette=6125,
        --tameable=true, -- hopper
        note="Fish up 10x{item:211474:Shadowblind Grouper} and throw them to fill the {spell:437124:Craving} bar. Objective of {questname:76588}.",
    },
    [57304858] = { -- Pride of Beledar
        criteria=69715,
        quest=81882, -- 84068
        npc=221786,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84068}),
            221225, -- Benevolent Hornstag Cinch
            221246, -- Fierce Beast Staff
            221247, -- Cavernous Critter Shooter
            221251, -- Bestial Underground Cleaver
            221265, -- Charm of the Underground Beast
            223007, -- Lance of Beledar's Pride
        },
        vignette=6159,
        -- tameable=true, -- stag
    },
    [67182424] = { -- Toadstomper
        criteria=69723,
        quest=82561, -- 84055
        npc=207803,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84055}),
            223920, -- Slime Deflecting Stopper
            223921, -- Ever-Oozing Signet
            223933, -- Slime Goliath's Cap
        },
        vignette=6084,
        note="Objective of {questname:76588}",
    },
    [64802920] = { -- Crazed Cabbage Smacker
        criteria=69720,
        quest=82558, -- 84052
        npc=206514,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84052}),
            211968, -- Blueprint Bundle
            221238, -- Pillar of Constructs
            223928, -- Crop Cutter's Gauntlets
            223935, -- Cabbage Harvester's Pantaloons
        },
        vignette=6120,
        note="Objective of {questname:76588}",
    },
    [60201860] = { -- Finclaw Bloodtide
        criteria=69727,
        quest=82564,
        npc=207780, -- also 220492?
        loot={
            221233, -- Deephunter's Bloody Hook
            221234, -- Tidal Pendant
            221248, -- Deep Terror Carver
            223925, -- Blood Hungerer's Chestplate
        },
        vignette=6085,
        note="Objective of {questname:76588}",
    },
    [62033212] = { -- Ravageant
        criteria=69726,
        quest=82566, -- 84058
        npc=207826,
        loot={
            ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=84058}),
            221240, -- Nerubian Stagshell Gouger
            221252, -- Nerubian Slayer's Claymore
            221263, -- Nerubian Venom-Tipped Dart
            223932, -- Scarab's Carapace Cap
        },
        vignette=6124,
        note="Objective of {questname:76588}",
    },
    [61403220] = { -- Parasidious
        criteria=69725,
        quest=82563,
        npc=206977,
        loot={
            221250, -- Creeping Lasher Machete
            221264, -- Fungarian Mystic's Cluster
            223005, -- String of Fungal Fruits
            223940, -- Deranged Fungarian's Epaulets
        },
        vignette=6361,
        note="Objective of {questname:76588}",
    },
    -- UNKNOWN LOCATION
    --[[
    [] = { -- Brineslash
        criteria=69718,
        quest=80486,
        npc=220159,
        vignette=6075,
    },
    --]]
}, {
    achievement=40851,
})
-- Beledar's Spawn
ns.RegisterPoints(ns.HALLOWFALL, {
    [25825754] = {},
    [32673962] = {},
    [37207191] = {},
    [37744585] = {},
    [38382474] = {},
    [42733133] = {},
    [45252569] = {},
    [47015504] = {},
    [48853197] = {},
    [50514857] = {},
    [51427080] = {},
    [54833679] = {},
    [58034885] = {},
    [60451862] = {},
    [61380753] = {},
    [62823857] = {},
    [68123014] = {},
    [71976558] = {},
    [72066566] = {},
    [72804152] = {},
}, {
    achievement=40851,
    criteria=69716,
    quest=81763, -- 85164
    npc=207802,
    loot={
        ns.rewards.Currency(ns.CURRENCY_ARATHI, 150, {quest=85164}),
        {223315, mount=2192,}, -- Beledar's Spawn
        223006, -- Signet of Dark Horizons
    },
    active={SHADOWPHASE},
    note="Spawns during the shadow event, which happens every 3 hours.\nBuy and use {item:224553:Beledar's Attunement} from {majorfaction:2570:Hallowfall Arathi} to see which spawn is active.",
    atlas="worldquest-icon-boss-zhCN",
    group="beledarspawn",
    vignette=6359, -- also 6118? That was the close-up one...
})

-- Deathtide
local deathtide = ns.nodeMaker{
    achievement=40851,
    criteria=69717,
    quest=81880,
    level=80, -- required to loot the offering/jar
}
ns.RegisterPoints(ns.HALLOWFALL, {
    [44744241] = { -- Deathtide
        npc=221753,
        loot={
            223920, -- Slime Deflecting Stopper
            223921, -- Ever-Oozing Signet
            225997, -- Earthen Adventurer's Spaulders
        },
        vignette=6156,
        active=ns.conditions.Item(220123), -- Ominous Offering
        note="Create an {item:220123:Ominous Offering} from {item:220124:Jar of Mucus} (|A:playerpartyblip:::::0:255:127|a) + {item:220122} (|A:playerpartyblip:::::0:0:255|a) to summon",
    },
}, deathtide{})
ns.RegisterPoints(ns.HALLOWFALL, {
    -- Jar of Mucus
    [48001668] = {route={48001668, 44744241, highlightOnly=true}},
}, deathtide{
    label="{item:220124}",
    loot={220124},
    texture=ns.atlas_texture("playerpartyblip",{r=0,g=1,b=0.5,}),
    minimap=true,
    note="Take to {npc:221753} @ 44.7,42.4",
})
ns.RegisterPoints(ns.HALLOWFALL, {
     -- Offering of Pure Water
    [28925120] = {route={28925120, 44744241, highlightOnly=true}},
    [34185782] = {route={34185782, 44744241, highlightOnly=true}},
    [34365357] = {route={34365357, 44744241, highlightOnly=true}},
    [43451413] = {route={43451413, 44744241, highlightOnly=true}},
    [50094966] = {route={50094966, 44744241, highlightOnly=true}},
    [53771913] = {route={53771913, 44744241, highlightOnly=true}},
    [55142344] = {route={55142344, 44744241, highlightOnly=true}},
}, deathtide{
    label="{item:220122}",
    loot={220122},
    texture=ns.atlas_texture("playerpartyblip",{r=0,g=0,b=1,}),
    minimap=true,
    note="Take to {npc:221753} @ 44.7,42.4",
})


-- ns.RegisterPoints(ns.HALLOWFALL, {
--     [62650611] = { -- Radiant-Twisted Mycelium
--         quest=nil, -- confirmed, this has a vignette and is rare-flagged, but no quest or rep rewards
--         npc=214905,
--         vignette=5984,
--         note="Objective of {questname:76588}",
--         additional={61953305},
--     },
-- })
