------------------
-- WoW API Events
------------------
local frame=CreateFrame("Frame")
-- Loading into the game, reloading, finishing cinematics, etc.
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
-- Cinematic Start
frame:RegisterEvent("CINEMATIC_START")
-- Vehicle Nonsense
frame:RegisterEvent("UNIT_ENTERING_VEHICLE")
frame:RegisterEvent("UNIT_ENTERED_VEHICLE")
frame:RegisterEvent("UNIT_EXITING_VEHICLE")
frame:RegisterEvent("UNIT_EXITED_VEHICLE")
-- For players joining / transitioning instances with same group
frame:RegisterEvent("GROUP_ROSTER_UPDATE")
frame:RegisterEvent("PLAYER_ROLES_ASSIGNED")
-- Combat Status ON/OFF
frame:RegisterEvent("PLAYER_REGEN_ENABLED")
frame:RegisterEvent("PLAYER_REGEN_DISABLED")
-- Rest status Updated
frame:RegisterEvent("PLAYER_UPDATE_RESTING")
-- Encounters starting/ending
frame:RegisterEvent("ENCOUNTER_START")
frame:RegisterEvent("ENCOUNTER_END")
-- Client Scene
frame:RegisterEvent("CLIENT_SCENE_OPENED")
frame:RegisterEvent("CLIENT_SCENE_CLOSED")


-- Player Spec Changed
frame:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED", "player")
-- Unregister Buffs since we are using our own tracker
frame:UnregisterEvent("UNIT_AURA")


-------------------------
--Unit Frame Creation Section
-------------------------

-- bless you, semlar: https://www.wowinterface.com/forums/showpost.php?p=311820&postcount=2
-- Create a new Player frame!!!
local button = CreateFrame('Button', 'ZUF_PlayerFrame', UIParent, 'SecureUnitButtonTemplate')
button:SetSize(200, 55)
button:SetPoint('CENTER', 1, -296)
button:RegisterForClicks('AnyUp') -- only reacts to left clicks by default
button:SetAttribute('unit', 'player') -- sets the associated unit
button:SetAttribute('*type1', 'target') -- makes left click target the unit
button:SetAttribute('*type2', 'togglemenu') -- makes right click toggle a unit menu

-- Do a Mouseover Tooltip!
button:SetScript("OnEnter", function(self, motion)
	-- Hook into the default GameTooltip for the unit!
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	GameTooltip:SetUnit("player")
	GameTooltip:Show()
end)
button:SetScript("OnLeave", function(self, motion)
	GameTooltip:FadeOut()
end)




-- Create a new Target frame!!!
local button = CreateFrame('Button', 'ZUF_TargetFrame', UIParent, 'SecureUnitButtonTemplate')

button:SetSize(200, 45)
button:SetPoint('TOP', 'ZUF_PlayerFrame', 'TOP', 0, 56)
button:RegisterForClicks('AnyUp') -- only reacts to left clicks by default
button:SetAttribute('unit', 'target') -- sets the associated unit
button:SetAttribute('*type1', 'target') -- makes left click target the unit
button:SetAttribute('*type2', 'togglemenu') -- makes right click toggle a unit menu
RegisterUnitWatch(button) -- shows/hides this button based on whether the associated unit exists or not

-- Do a Mouseover Tooltip!
button:SetScript("OnEnter", function(self, motion)
	-- Hook into the default GameTooltip for the unit!
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	GameTooltip:SetUnit("target")
	GameTooltip:Show()
end)
button:SetScript("OnLeave", function(self, motion)
	GameTooltip:FadeOut()
end)


-- Create a new Target of Target frame!!!
local button = CreateFrame('Button', 'ZUF_ToT', UIParent, 'SecureUnitButtonTemplate')

button:SetSize(82, 40)
button:SetPoint('RIGHT', 'ZUF_TargetFrame', 'RIGHT', 89, 0)
button:RegisterForClicks('AnyUp') -- only reacts to left clicks by default
button:SetAttribute('unit', 'targettarget') -- sets the associated unit
button:SetAttribute('*type1', 'target') -- makes left click target the unit
button:SetAttribute('*type2', 'togglemenu') -- makes right click toggle a unit menu
RegisterUnitWatch(button) -- shows/hides this button based on whether the associated unit exists or not

-- Do a Mouseover Tooltip!
button:SetScript("OnEnter", function(self, motion)
	-- Hook into the default GameTooltip for the unit!
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	GameTooltip:SetUnit("targettarget")
	GameTooltip:Show()
end)
button:SetScript("OnLeave", function(self, motion)
	GameTooltip:FadeOut()
end)


-- Create a new Focus frame!!!
local button = CreateFrame('Button', 'ZUF_FocusFrame', UIParent, 'SecureUnitButtonTemplate')

button:SetSize(82, 40)
button:SetPoint('RIGHT', 'ZUF_ToT', 'RIGHT', 88, 0)
button:RegisterForClicks('AnyUp') -- only reacts to left clicks by default
button:SetAttribute('unit', 'focus') -- sets the associated unit
button:SetAttribute('*type1', 'target') -- makes left click target the unit
button:SetAttribute('*type2', 'togglemenu') -- makes right click toggle a unit menu
RegisterUnitWatch(button) -- shows/hides this button based on whether the associated unit exists or not

-- Do a Mouseover Tooltip!
button:SetScript("OnEnter", function(self, motion)
	-- Hook into the default GameTooltip for the unit!
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	GameTooltip:SetUnit("focus")
	GameTooltip:Show()
end)
button:SetScript("OnLeave", function(self, motion)
	GameTooltip:FadeOut()
end)

---------------------
-- Script Section
---------------------

-- START create the main zeraUFScripts function, does the stuff we want and we can re-call it during normal events as well as hooksecurefunc stuff (so we don't have to redundantly script everything)
local function zeraUFScripts()   
    
    -- Hide stock Player Frame
    PlayerFrame:UnregisterAllEvents()
    PlayerFrame:ClearAllPoints()
    PlayerFrame:Hide()
	-- Hide Player Frame Container Textures
	PlayerFrame.PlayerFrameContainer:Hide()
	PlayerFrame.PlayerFrameContainer:UnregisterAllEvents();
	PlayerFrame.PlayerFrameContainer:ClearAllPoints()
	-- Unregister events for player frame animations (resting, hits, etc.)
	PlayerFrame.PlayerFrameContent:Hide()
	PlayerFrame.PlayerFrameContent:UnregisterAllEvents();
	PlayerFrame.PlayerFrameContent:ClearAllPoints()
    -- Hide stock Resource frame
    PlayerFrameBottomManagedFramesContainer:Hide()
    -- Unregister the Player Casting Bar
    PlayerCastingBarFrame:UnregisterAllEvents()
    -- Hide Stock Target Frame
    TargetFrame:UnregisterAllEvents()
    TargetFrame:ClearAllPoints()
	TargetFrame:Hide()
    -- Unregister the Target Casting Bar
    TargetFrameSpellBar:UnregisterAllEvents()    
    -- Hide stock Target of Target frame
    TargetFrameToT:UnregisterAllEvents()
    TargetFrameToT:ClearAllPoints()
	TargetFrameToT:Hide()
    --Hide stock Focus Frame
    FocusFrame:UnregisterAllEvents()
    FocusFrame:ClearAllPoints()
    FocusFrame:Hide()
    --Hide Focus Cast Bar
    FocusFrameSpellBar:UnregisterAllEvents()
    -- Clear All Points then Unregister the Focus Frame Target of Target
    FocusFrameToT:Hide()
    FocusFrameToT:UnregisterAllEvents()
    FocusFrameToT:ClearAllPoints()
    -- Hide Default Debuff Frame
    DebuffFrame:Hide()
	
	-- Conditional to check Classes that have Resource Bars and move the Player Frame down slightly if so!	
	-- First, we get the Specialization Number (returns 1, 2, 3, or 4 for all Classes)
	local specNumber = GetSpecialization()

	-- Then, we conditionally check the Class (built into API via UnitClass) and check the Spec number we called above
	if UnitClass("Player") == "Paladin" 
	or UnitClass("Player") == "Rogue"
	or UnitClass("Player") == "Death Knight"
	-- Monks, except Mistweaver
	or UnitClass("Player") == "Monk" and specNumber ~= 2
	or UnitClass("Player") == "Warlock"
	or UnitClass("Player") == "Druid"
	-- Mages, but only Arcane
	or UnitClass("Player") == "Mage" and specNumber == 1
	or UnitClass("Player") == "Evoker" then 
		ZUF_PlayerFrame:SetPoint('CENTER', 1, -306)
		ZUF_TargetFrame:SetPoint('TOP', 'ZUF_PlayerFrame', 'TOP', 0, 56)
	else 
		ZUF_PlayerFrame:SetPoint('CENTER', 1, -296)
		ZUF_TargetFrame:SetPoint('TOP', 'ZUF_PlayerFrame', 'TOP', 0, 46)
	end
	-- That's it, the Player Frame moves slightly based on specs having/not having Resource Bars :D

end

-- START Main standard WoW script function for events
frame:SetScript("OnEvent",function(self,event,...)
        
        -- call the scripts we want into the standward WoW script function for events
        zeraUFScripts()
        
        
        
        
        -- END Main standard WoW script function for events
end); 


